@@ -33,7 +33,11 @@
 #include <xmlrpc-c/base.h>
 #include <xmlrpc-c/client.h>
 #include <pthread.h>
+#ifndef ANDROID
 #include <glib.h>
+#else
+#include "android/glib.h"
+#endif
 #include <pixman.h>
 #include <cairo.h>
 
@@ -99,6 +103,7 @@ struct chamelium_port {
 	char *name;
 	bool adapter_allowed;
 	char *connector_path;
+	bool is_mapped;
 };
 
 struct chamelium_frame_dump {
@@ -152,7 +157,7 @@ struct chamelium_port **chamelium_get_ports(struct chamelium *chamelium,
 {
 	int i;
 	struct chamelium_port **ret =
-		calloc(sizeof(void*), chamelium->port_count);
+		calloc(chamelium->port_count, sizeof(void*));
 
 	*count = chamelium->port_count;
 	for (i = 0; i < chamelium->port_count; i++)
@@ -523,6 +528,11 @@ static bool wait_for_connected_state(int drm_fd,
 			drmModeConnector *connector =
 				drmModeGetConnector(drm_fd, connectors[i]);
 
+			if (!connector) {
+				connected = false;
+				break;
+			}
+
 			connected = connector->connection == DRM_MODE_CONNECTED;
 
 			drmModeFreeConnector(connector);
@@ -1515,7 +1525,7 @@ igt_crc_t *chamelium_read_captured_crcs(struct chamelium *chamelium,
 	res = chamelium_rpc(chamelium, NULL, "GetCapturedChecksums", "(in)", 0);
 
 	*frame_count = xmlrpc_array_size(&chamelium->env, res);
-	ret = calloc(sizeof(igt_crc_t), *frame_count);
+	ret = calloc(*frame_count, sizeof(igt_crc_t));
 
 	for (i = 0; i < *frame_count; i++) {
 		xmlrpc_array_read_item(&chamelium->env, res, i, &elem);
@@ -1872,7 +1882,7 @@ static void compared_frames_dump(cairo_surface_t *reference,
 	}
 
 	if (!capture_crc) {
-		chamelium_do_calculate_fb_crc(reference, &local_capture_crc);
+		chamelium_do_calculate_fb_crc(capture, &local_capture_crc);
 		capture_crc = &local_capture_crc;
 	}
 
@@ -2064,6 +2074,52 @@ bool chamelium_frame_match_or_dump(struct chamelium *chamelium,
 	return match;
 }
 
+/**
+ * chamelium_frame_match_or_dump_frame_pair:
+ * @chamelium: The chamelium instance the frame dump belongs to
+ * @frame0: The chamelium reference frame dump to match
+ * @frame1: The chamelium capture frame dump to match
+ * @check: the type of frame matching check to use
+ *
+ * Returns bool that the provided captured frames matches.
+ * If they do not, this saves the frames to a png file.
+ */
+bool chamelium_frame_match_or_dump_frame_pair(struct chamelium *chamelium,
+					      struct chamelium_port *port,
+					      const struct chamelium_frame_dump *frame0,
+					      const struct chamelium_frame_dump *frame1,
+					      enum chamelium_check check)
+{
+	cairo_surface_t *reference;
+	cairo_surface_t *capture;
+	bool match;
+
+	/* Grab the captured reference frame from chamelium */
+	reference = convert_frame_dump_argb32(frame0);
+
+	/* Grab the captured frame from chamelium */
+	capture = convert_frame_dump_argb32(frame1);
+
+	switch (check) {
+	case CHAMELIUM_CHECK_ANALOG:
+		match = igt_check_analog_frame_match(reference, capture);
+		break;
+	case CHAMELIUM_CHECK_CHECKERBOARD:
+		match = igt_check_checkerboard_frame_match(reference, capture);
+		break;
+	default:
+		igt_assert(false);
+	}
+
+	if (!match && igt_frame_dump_is_enabled())
+		compared_frames_dump(reference, capture, 0, 0);
+
+	cairo_surface_destroy(reference);
+	cairo_surface_destroy(capture);
+
+	return match;
+}
+
 /**
  * chamelium_analog_frame_crop:
  * @chamelium: The Chamelium instance to use
@@ -2637,6 +2693,43 @@ static bool get_connector_id_for_port(struct chamelium *chamelium,
 	return false;
 }
 
+static void selective_sort_ports(struct chamelium_port *ports) {
+    int mapped_count = 0;
+    int unmapped_count = 0;
+    int mapped_index = 0;
+    int unmapped_index = mapped_count;
+    struct chamelium_port sorted_ports[CHAMELIUM_MAX_PORTS];
+
+    /*
+     * Count the number of mapped and unmapped ports
+     */
+    for (int i = 0; i < CHAMELIUM_MAX_PORTS; i++) {
+        if (ports[i].is_mapped) {
+            mapped_count++;
+        } else {
+            unmapped_count++;
+        }
+    }
+
+    /*
+     * Rearrange the ports such that mapped ports are at the start
+     */
+    unmapped_index = mapped_count;
+
+    for (int i = 0; i < CHAMELIUM_MAX_PORTS; i++) {
+        if (ports[i].is_mapped) {
+            sorted_ports[mapped_index++] = ports[i];
+        } else {
+            sorted_ports[unmapped_index++] = ports[i];
+        }
+    }
+
+    /*
+     * Copy the sorted ports back to the original array
+     */
+    memcpy(ports, sorted_ports, CHAMELIUM_MAX_PORTS * sizeof(struct chamelium_port));
+}
+
 /**
  * chamelium_autodiscover: automagically discover the Chamelium port mapping
  *
@@ -2721,6 +2814,7 @@ static bool chamelium_autodiscover(struct chamelium *chamelium)
 			igt_info("Failed to auto-discover port %d\n", port_id);
 			goto unplug_port;
 		}
+		port->is_mapped = true;
 		is_any_port_mapped = true;
 
 		/* If we found a connector for our port, increment the number of valid Chamelium ports. */
@@ -2746,12 +2840,17 @@ unplug_port:
 	}
 
 	/* After we're all set, turn on all supported ports */
-	for (i = 0; i < chamelium->port_count; i++) {
+	for (i = 0; i < CHAMELIUM_MAX_PORTS; i++) {
 		struct chamelium_port *port = &chamelium->ports[i];
-		chamelium_plug(chamelium, port);
+		if (port->is_mapped)
+			chamelium_plug(chamelium, port);
 	}
 	sleep(CHAMELIUM_HOTPLUG_DETECTION_DELAY);
 
+	/*
+	 * Sort the ports based on the is_mapped flag
+	 */
+	selective_sort_ports(chamelium->ports);
 	elapsed_ns = igt_nsec_elapsed(&start);
 	igt_debug("Auto-discovery took %fms and found %i connector(s)\n",
 		  (float)elapsed_ns / (1000 * 1000), chamelium->port_count);
@@ -2903,6 +3002,13 @@ struct chamelium *chamelium_init(int drm_fd, igt_display_t *display)
 	chamelium->drm_fd = dup(drm_fd);
 	IGT_INIT_LIST_HEAD(&chamelium->edids);
 
+	/*
+	 * Reset the chamelium and do a disabling modeset for
+	 * TypeC connector to come up nicely
+	 */
+	chamelium_reset(chamelium);
+	igt_modeset_disable_all_outputs(display);
+
 	if (!chamelium_read_port_mappings(chamelium, drm_fd))
 		goto error;
 
