@@ -46,8 +46,9 @@
 #include "igt_core.h"
 #include "igt_sysfs.h"
 #include "igt_device.h"
-#include "igt_io.h"
+#include "igt_fs.h"
 #include "intel_chipset.h"
+#include "xe/xe_query.h"
 
 /**
  * SECTION:igt_sysfs
@@ -225,6 +226,7 @@ int igt_sysfs_open(int device)
 char *xe_sysfs_gt_path(int xe_device, int gt, char *path, int pathlen)
 {
 	struct stat st;
+	struct xe_device *xe_dev;
 
 	if (xe_device < 0)
 		return NULL;
@@ -232,12 +234,12 @@ char *xe_sysfs_gt_path(int xe_device, int gt, char *path, int pathlen)
 	if (igt_debug_on(fstat(xe_device, &st)) || igt_debug_on(!S_ISCHR(st.st_mode)))
 		return NULL;
 
-	if (IS_PONTEVECCHIO(intel_get_drm_devid(xe_device)))
-		snprintf(path, pathlen, "/sys/dev/char/%d:%d/device/tile%d/gt%d",
-			 major(st.st_rdev), minor(st.st_rdev), gt, gt);
-	else
-		snprintf(path, pathlen, "/sys/dev/char/%d:%d/device/tile0/gt%d",
-			 major(st.st_rdev), minor(st.st_rdev), gt);
+	xe_dev = xe_device_get(xe_device);
+
+	igt_assert(xe_dev);
+
+	snprintf(path, pathlen, "/sys/dev/char/%d:%d/device/tile%d/gt%d",
+		 major(st.st_rdev), minor(st.st_rdev), xe_get_tile(xe_dev, gt), gt);
 
 	if (!access(path, F_OK))
 		return path;
@@ -291,22 +293,6 @@ bool xe_sysfs_gt_has_node(int xe_device, int gt, const char *node)
 	return has_node;
 }
 
-static const char *xe_engine_class_to_str(__u16 class)
-{
-	static const char * const str[] = {
-		[DRM_XE_ENGINE_CLASS_RENDER] = "rcs",
-		[DRM_XE_ENGINE_CLASS_COPY] = "bcs",
-		[DRM_XE_ENGINE_CLASS_VIDEO_DECODE] = "vcs",
-		[DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE] = "vecs",
-		[DRM_XE_ENGINE_CLASS_COMPUTE] = "ccs",
-	};
-
-	if (class < ARRAY_SIZE(str))
-		return str[class];
-
-	return "unk";
-}
-
 /**
  * xe_sysfs_engine_path:
  * @xe_device: fd of the device
@@ -322,7 +308,7 @@ char *
 xe_sysfs_engine_path(int xe_device, int gt, int class, char *path, int pathlen)
 {
 	struct stat st;
-	int tile = IS_PONTEVECCHIO(intel_get_drm_devid(xe_device)) ? gt : 0;
+	char base_path[96];
 
 	if (xe_device < 0)
 		return NULL;
@@ -330,8 +316,9 @@ xe_sysfs_engine_path(int xe_device, int gt, int class, char *path, int pathlen)
 	if (igt_debug_on(fstat(xe_device, &st)) || igt_debug_on(!S_ISCHR(st.st_mode)))
 		return NULL;
 
-	snprintf(path, pathlen, "/sys/dev/char/%d:%d/device/tile%d/gt%d/engines/%s",
-		 major(st.st_rdev), minor(st.st_rdev), tile, gt, xe_engine_class_to_str(class));
+	xe_sysfs_gt_path(xe_device, gt, base_path, sizeof(base_path));
+	snprintf(path, pathlen, "%s/engines/%s", base_path,
+		 xe_engine_class_short_string(class));
 
 	if (!access(path, F_OK))
 		return path;
@@ -457,23 +444,23 @@ int igt_sysfs_drm_module_params_open(void)
 	return open(path, O_RDONLY);
 }
 
-static int log_level = -1;
+static int saved_drm_debug_mask = -1;
 
 /**
- * igt_drm_debug_level_get:
+ * igt_drm_debug_mask_get:
  *
- * This reads the current debug log level of the machine on
+ * This reads the current debug mask of the machine on
  * which the test is currently executing.
  *
  * Returns:
- * The current log level, or -1 on error.
+ * The current debug mask, or -1 on error.
  */
-int igt_drm_debug_level_get(int dir)
+int igt_drm_debug_mask_get(int dir)
 {
 	char buf[20];
 
-	if (log_level >= 0)
-		return log_level;
+	if (saved_drm_debug_mask >= 0)
+		return saved_drm_debug_mask;
 
 	if (igt_sysfs_read(dir, "debug", buf, sizeof(buf) - 1) < 0)
 		return -1;
@@ -482,44 +469,45 @@ int igt_drm_debug_level_get(int dir)
 }
 
 /**
- * igt_drm_debug_level_reset:
+ * igt_drm_debug_mask_reset:
  *
- * This modifies the current debug log level of the machine
+ * This modifies the current debug mask of the machine
  * to the default value post-test.
  *
  */
-void igt_drm_debug_level_reset(void)
+void igt_drm_debug_mask_reset(void)
 {
 	char buf[20];
 	int dir;
 
-	if (log_level < 0)
+	if (saved_drm_debug_mask < 0)
 		return;
 
 	dir = igt_sysfs_drm_module_params_open();
 	if (dir < 0)
 		return;
 
-	igt_debug("Resetting DRM debug level to %d\n", log_level);
-	snprintf(buf, sizeof(buf), "%d", log_level);
+	igt_debug("Restoring DRM debug mask to %d\n", saved_drm_debug_mask);
+	snprintf(buf, sizeof(buf), "%d", saved_drm_debug_mask);
 	igt_assert(igt_sysfs_set(dir, "debug", buf));
 
 	close(dir);
 }
 
-static void igt_drm_debug_level_reset_exit_handler(int sig)
+void igt_drm_debug_mask_reset_exit_handler(int sig)
 {
-	igt_drm_debug_level_reset();
+	igt_drm_debug_mask_reset();
 }
 
 /**
- * igt_drm_debug_level_update:
- * @debug_level: new debug level to set
+ * igt_drm_debug_mask_update:
+ * @mask_to_set: new debug mask to set
  *
- * This modifies the current drm debug log level to the new value.
+ * This modifies the current drm debug mask to the new value.
  */
-void igt_drm_debug_level_update(unsigned int new_log_level)
+void igt_drm_debug_mask_update(unsigned int mask_to_set)
 {
+	static bool debug_mask_read_once = true;
 	char buf[20];
 	int dir;
 
@@ -527,23 +515,31 @@ void igt_drm_debug_level_update(unsigned int new_log_level)
 	if (dir < 0)
 		return;
 
-	log_level = igt_drm_debug_level_get(dir);
-	if (log_level < 0) {
-		close(dir);
-		return;
+	/* The below flag is used to read the original debug mask only once */
+	if (debug_mask_read_once) {
+		debug_mask_read_once = false;
+		saved_drm_debug_mask = igt_drm_debug_mask_get(dir);
+		if (saved_drm_debug_mask < 0) {
+			close(dir);
+			return;
+		}
 	}
 
-	igt_debug("Setting DRM debug level to %d\n", new_log_level);
-	snprintf(buf, sizeof(buf), "%d", new_log_level);
+	igt_debug("Setting DRM debug mask to %d\n", mask_to_set);
+	snprintf(buf, sizeof(buf), "%d", mask_to_set);
 	igt_assert(igt_sysfs_set(dir, "debug", buf));
 
 	close(dir);
+}
 
-	/*
-	 * TODO: Check whether multiple exit handlers will get installed,
-	 * if we call this api multiple times
-	 */
-	igt_install_exit_handler(igt_drm_debug_level_reset_exit_handler);
+void update_debug_mask_if_ci(unsigned int debug_mask_if_ci)
+{
+	const char *ci_run = getenv("IGT_CI_RUN");
+
+	if (ci_run && ci_run[0] == '1') {
+		igt_debug("Currently under CI execution, reducing the DRM debug mask to 0x4\n");
+		igt_drm_debug_mask_update(debug_mask_if_ci);
+	}
 }
 
 /**
@@ -873,6 +869,77 @@ void igt_sysfs_set_u32(int dir, const char *attr, uint32_t value)
 		     "Failed to write %u to %s attribute (%s)\n", value, attr, strerror(errno));
 }
 
+/**
+ * __igt_sysfs_get_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to read
+ * @value: pointer for storing read value
+ *
+ * Convenience wrapper to read a signed 32bit integer from a sysfs file.
+ *
+ * Returns:
+ * True if value successfully read, false otherwise.
+ */
+bool __igt_sysfs_get_s32(int dir, const char *attr, int32_t *value)
+{
+	if (igt_debug_on(igt_sysfs_scanf(dir, attr, "%d", value) != 1))
+		return false;
+
+	return true;
+}
+
+/**
+ * igt_sysfs_get_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to read
+ *
+ * Convenience wrapper to read a signed 32bit integer from a sysfs file.
+ * It asserts on failure.
+ *
+ * Returns:
+ * Read value.
+ */
+int32_t igt_sysfs_get_s32(int dir, const char *attr)
+{
+	int32_t value;
+
+	igt_assert_f(__igt_sysfs_get_s32(dir, attr, &value),
+		     "Failed to read %s attribute (%s)\n", attr, strerror(errno));
+
+	return value;
+}
+
+/**
+ * __igt_sysfs_set_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to write
+ * @value: value to set
+ *
+ * Convenience wrapper to write a signed 32bit integer to a sysfs file.
+ *
+ * Returns:
+ * True if successfully written, false otherwise.
+ */
+bool __igt_sysfs_set_s32(int dir, const char *attr, int32_t value)
+{
+	return igt_sysfs_printf(dir, attr, "%d", value) > 0;
+}
+
+/**
+ * igt_sysfs_set_s32:
+ * @dir: directory corresponding to attribute
+ * @attr: name of the sysfs node to write
+ * @value: value to set
+ *
+ * Convenience wrapper to write a signed 32bit integer to a sysfs file.
+ * It asserts on failure.
+ */
+void igt_sysfs_set_s32(int dir, const char *attr, int32_t value)
+{
+	igt_assert_f(__igt_sysfs_set_s32(dir, attr, value),
+		     "Failed to write %d to %s attribute (%s)\n", value, attr, strerror(errno));
+}
+
 /**
  * __igt_sysfs_get_u64:
  * @dir: directory corresponding to attribute
@@ -1152,7 +1219,7 @@ static int rw_attr_sweep(igt_sysfs_rw_attr_t *rw)
 	while (set < UINT64_MAX / 2) {
 		ret = __igt_sysfs_set_u64(rw->dir, rw->attr, set);
 		__igt_sysfs_get_u64(rw->dir, rw->attr, &get);
-		igt_debug("'%s': ret %d set %lu get %lu\n", rw->attr, ret, set, get);
+		igt_debug("'%s': ret %d set %"PRIu64" get %"PRIu64"\n", rw->attr, ret, set, get);
 		if (ret && rw_attr_equal_within_epsilon(get, set, rw->tol)) {
 			igt_debug("'%s': matches\n", rw->attr);
 			num_points++;
@@ -1196,7 +1263,7 @@ void igt_sysfs_rw_attr_verify(igt_sysfs_rw_attr_t *rw)
 	igt_assert(rw->start);	/* cannot be 0 */
 
 	__igt_sysfs_get_u64(rw->dir, rw->attr, &prev);
-	igt_debug("'%s': prev %lu\n", rw->attr, prev);
+	igt_debug("'%s': prev %"PRIu64"\n", rw->attr, prev);
 
 	ret = rw_attr_sweep(rw);
 
@@ -1236,6 +1303,67 @@ static uint16_t xe_get_engine_class(char *name)
 	return class;
 }
 
+/**
+ * igt_sysfs_get_engine_list:
+ * @engines: fd of the directory engine
+ *
+ * Iterates over sysfs/engines and returns an array of
+ * opened engines.  The user will be in charge of closing
+ * the opened engines.
+ *
+ * The returned array will always be terminated by a -1.
+ */
+int *igt_sysfs_get_engine_list(int engines)
+{
+	struct dirent *de;
+	DIR *dir;
+	const int array_max = 16;
+	int *ret = calloc(array_max, sizeof(int));
+	int size = 0;
+
+	igt_assert(ret);
+
+	lseek(engines, 0, SEEK_SET);
+
+	dir = fdopendir(engines);
+	if (!dir)
+		close(engines);
+
+	while ((de = readdir(dir))) {
+		if (*de->d_name == '.')
+			continue;
+		igt_assert_lt(size, array_max);
+		ret[size] = openat(engines, de->d_name, O_RDONLY);
+		if (ret[size] < 0) {
+			ret[size] = 0;
+			continue;
+		}
+		size += 1;
+	}
+
+	igt_assert_lt(size, array_max);
+	ret[size] = -1;
+
+	return ret;
+}
+
+/**
+ * igt_sysfs_free_engine_list:
+ * @list: list of opened engines
+ * @size: number of engines in list
+ *
+ * Helper for cleaning up after igt_sysfs_get_engine_list.
+ * Closes all engines in list before freeing the list.
+ */
+void igt_sysfs_free_engine_list(int *list)
+{
+	int i = 0;
+
+	while (list[i] != -1)
+		close(list[i++]);
+	free(list);
+}
+
 /**
  * igt_sysfs_engines:
  * @xe: fd of the device
@@ -1360,3 +1488,88 @@ int xe_sysfs_get_num_tiles(int xe_device)
 
 	return num_tiles;
 }
+
+/**
+ * xe_sysfs_engine_class_get_property
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ * @property: property of engine class to retrieve
+ * @value: pointer for storing read value
+ *
+ * Convenience wrapper to get value of given property for given engine class on given gt.
+ *
+ * Returns: true on success, false on failure.
+ */
+bool xe_sysfs_engine_class_get_property(int xe_device, int gt, uint16_t class, const char *property,
+					uint32_t *value)
+{
+	int engines_fd;
+
+	engines_fd = xe_sysfs_engine_open(xe_device, gt, class);
+
+	if (engines_fd == -1) {
+		igt_debug("Failed to open %s on gt%d.\n", xe_engine_class_short_string(class), gt);
+
+		return false;
+	}
+
+	if (!__igt_sysfs_get_u32(engines_fd, property, value)) {
+		igt_debug("Failed to read %s property of %s on gt%d.\n", property,
+			  xe_engine_class_short_string(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	close(engines_fd);
+
+	return true;
+}
+
+/**
+ * xe_sysfs_engine_class_set_property
+ * @xe_device: fd of the device
+ * @gt: gt number
+ * @class: engine class
+ * @property: property of engine class to be modified
+ * @new_value: value to be set
+ * @old_value: pointer for storing old value, can be NULL
+ *
+ * Convenience wrapper to set given property for given engine class on given gt to given value.
+ *
+ * Returns: true on success, false on failure.
+ */
+bool xe_sysfs_engine_class_set_property(int xe_device, int gt, uint16_t class, const char *property,
+					uint32_t new_value, uint32_t *old_value)
+{
+	int engines_fd;
+
+	engines_fd = xe_sysfs_engine_open(xe_device, gt, class);
+
+	if (engines_fd == -1) {
+		igt_debug("Failed to open %s on gt%d.\n", xe_engine_class_short_string(class), gt);
+
+		return false;
+	}
+
+	if (old_value && !__igt_sysfs_get_u32(engines_fd, property, old_value)) {
+		igt_debug("Failed to read %s property of %s on gt%d.\n", property,
+			  xe_engine_class_short_string(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	if (!__igt_sysfs_set_u32(engines_fd, property, new_value)) {
+		igt_debug("Failed to write %s property of %s on gt%d.\n", property,
+			  xe_engine_class_short_string(class), gt);
+		close(engines_fd);
+
+		return false;
+	}
+
+	close(engines_fd);
+
+	return true;
+}
