@@ -9,6 +9,15 @@
 #include <stdlib.h>
 #include <pthread.h>
 
+#ifdef HAVE_VALGRIND
+#include <valgrind/valgrind.h>
+#include <valgrind/memcheck.h>
+
+#define VG(x) x
+#else
+#define VG(x) do {} while (0)
+#endif
+
 #include "drmtest.h"
 #include "ioctl_wrappers.h"
 #include "igt_map.h"
@@ -34,11 +43,42 @@ static struct drm_xe_query_config *xe_query_config_new(int fd)
 	query.data = to_user_pointer(config);
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(config, query.size));
+
 	igt_assert(config->num_params > 0);
 
 	return config;
 }
 
+static uint32_t *xe_query_hwconfig_new(int fd, uint32_t *hwconfig_size)
+{
+	uint32_t *hwconfig;
+	struct drm_xe_device_query query = {
+		.extensions = 0,
+		.query = DRM_XE_DEVICE_QUERY_HWCONFIG,
+		.size = 0,
+		.data = 0,
+	};
+
+	/* Perform the initial query to get the size */
+	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
+	if (!query.size)
+		return NULL;
+
+	hwconfig = malloc(query.size);
+	igt_assert(hwconfig);
+
+	query.data = to_user_pointer(hwconfig);
+
+	/* Perform the query to get the actual data */
+	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
+
+	VG(VALGRIND_MAKE_MEM_DEFINED(hwconfig, query.size));
+
+	*hwconfig_size = query.size;
+	return hwconfig;
+}
+
 static struct drm_xe_query_gt_list *xe_query_gt_list_new(int fd)
 {
 	struct drm_xe_query_gt_list *gt_list;
@@ -57,6 +97,8 @@ static struct drm_xe_query_gt_list *xe_query_gt_list_new(int fd)
 	query.data = to_user_pointer(gt_list);
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(gt_list, query.size));
+
 	return gt_list;
 }
 
@@ -90,6 +132,8 @@ static struct drm_xe_query_engines *xe_query_engines(int fd)
 	query.data = to_user_pointer(engines);
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(engines, query.size));
+
 	return engines;
 }
 
@@ -111,9 +155,37 @@ static struct drm_xe_query_mem_regions *xe_query_mem_regions_new(int fd)
 	query.data = to_user_pointer(mem_regions);
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(mem_regions, query.size));
+
 	return mem_regions;
 }
 
+static struct drm_xe_query_eu_stall *xe_query_eu_stall_new(int fd)
+{
+	struct drm_xe_query_eu_stall *query_eu_stall;
+	struct drm_xe_device_query query = {
+		.extensions = 0,
+		.query = DRM_XE_DEVICE_QUERY_EU_STALL,
+		.size = 0,
+		.data = 0,
+	};
+
+	/* Support older kernels where this uapi is not yet available */
+	if (igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query))
+		return NULL;
+	igt_assert_neq(query.size, 0);
+
+	query_eu_stall = malloc(query.size);
+	igt_assert(query_eu_stall);
+
+	query.data = to_user_pointer(query_eu_stall);
+	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
+
+	VG(VALGRIND_MAKE_MEM_DEFINED(query_eu_stall, query.size));
+
+	return query_eu_stall;
+}
+
 static struct drm_xe_query_oa_units *xe_query_oa_units_new(int fd)
 {
 	struct drm_xe_query_oa_units *oa_units;
@@ -134,24 +206,26 @@ static struct drm_xe_query_oa_units *xe_query_oa_units_new(int fd)
 	query.data = to_user_pointer(oa_units);
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 
+	VG(VALGRIND_MAKE_MEM_DEFINED(oa_units, query.size));
+
 	return oa_units;
 }
 
-static uint64_t native_region_for_gt(const struct drm_xe_query_gt_list *gt_list, int gt)
+static uint64_t native_region_for_gt(const struct drm_xe_gt *gt)
 {
 	uint64_t region;
 
-	igt_assert(gt_list->num_gt > gt);
-	region = gt_list->gt_list[gt].near_mem_regions;
+	igt_assert(gt);
+	region = gt->near_mem_regions;
 	igt_assert(region);
 
 	return region;
 }
 
 static uint64_t gt_vram_size(const struct drm_xe_query_mem_regions *mem_regions,
-			     const struct drm_xe_query_gt_list *gt_list, int gt)
+			     const struct drm_xe_gt *gt)
 {
-	int region_idx = ffs(native_region_for_gt(gt_list, gt)) - 1;
+	int region_idx = ffsll(native_region_for_gt(gt)) - 1;
 
 	if (XE_IS_CLASS_VRAM(&mem_regions->mem_regions[region_idx]))
 		return mem_regions->mem_regions[region_idx].total_size;
@@ -160,9 +234,9 @@ static uint64_t gt_vram_size(const struct drm_xe_query_mem_regions *mem_regions,
 }
 
 static uint64_t gt_visible_vram_size(const struct drm_xe_query_mem_regions *mem_regions,
-				     const struct drm_xe_query_gt_list *gt_list, int gt)
+				     const struct drm_xe_gt *gt)
 {
-	int region_idx = ffs(native_region_for_gt(gt_list, gt)) - 1;
+	int region_idx = ffsll(native_region_for_gt(gt)) - 1;
 
 	if (XE_IS_CLASS_VRAM(&mem_regions->mem_regions[region_idx]))
 		return mem_regions->mem_regions[region_idx].cpu_visible_size;
@@ -199,19 +273,44 @@ static uint32_t __mem_default_alignment(struct drm_xe_query_mem_regions *mem_reg
 const char *xe_engine_class_string(uint32_t engine_class)
 {
 	switch (engine_class) {
-		case DRM_XE_ENGINE_CLASS_RENDER:
-			return "DRM_XE_ENGINE_CLASS_RENDER";
-		case DRM_XE_ENGINE_CLASS_COPY:
-			return "DRM_XE_ENGINE_CLASS_COPY";
-		case DRM_XE_ENGINE_CLASS_VIDEO_DECODE:
-			return "DRM_XE_ENGINE_CLASS_VIDEO_DECODE";
-		case DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE:
-			return "DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE";
-		case DRM_XE_ENGINE_CLASS_COMPUTE:
-			return "DRM_XE_ENGINE_CLASS_COMPUTE";
-		default:
-			igt_warn("Engine class 0x%x unknown\n", engine_class);
-			return "unknown engine class";
+	case DRM_XE_ENGINE_CLASS_RENDER:
+		return "DRM_XE_ENGINE_CLASS_RENDER";
+	case DRM_XE_ENGINE_CLASS_COPY:
+		return "DRM_XE_ENGINE_CLASS_COPY";
+	case DRM_XE_ENGINE_CLASS_VIDEO_DECODE:
+		return "DRM_XE_ENGINE_CLASS_VIDEO_DECODE";
+	case DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE:
+		return "DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE";
+	case DRM_XE_ENGINE_CLASS_COMPUTE:
+		return "DRM_XE_ENGINE_CLASS_COMPUTE";
+	default:
+		igt_warn("Engine class 0x%x unknown\n", engine_class);
+		return "unknown engine class";
+	}
+}
+
+/**
+ * xe_engine_class_short_string:
+ * @engine_class: engine class
+ *
+ * Returns short name for engine class or 'unknown' otherwise.
+ */
+const char *xe_engine_class_short_string(uint32_t engine_class)
+{
+	switch (engine_class) {
+	case DRM_XE_ENGINE_CLASS_RENDER:
+		return "rcs";
+	case DRM_XE_ENGINE_CLASS_COPY:
+		return "bcs";
+	case DRM_XE_ENGINE_CLASS_VIDEO_DECODE:
+		return "vcs";
+	case DRM_XE_ENGINE_CLASS_VIDEO_ENHANCE:
+		return "vecs";
+	case DRM_XE_ENGINE_CLASS_COMPUTE:
+		return "ccs";
+	default:
+		igt_warn("Engine class 0x%x unknown\n", engine_class);
+		return "unknown";
 	}
 }
 
@@ -239,10 +338,12 @@ static struct xe_device *find_in_cache(int fd)
 static void xe_device_free(struct xe_device *xe_dev)
 {
 	free(xe_dev->config);
+	free(xe_dev->hwconfig);
 	free(xe_dev->gt_list);
 	free(xe_dev->engines);
 	free(xe_dev->mem_regions);
 	free(xe_dev->vram_size);
+	free(xe_dev->eu_stall);
 	free(xe_dev);
 }
 
@@ -258,6 +359,7 @@ static void xe_device_free(struct xe_device *xe_dev)
 struct xe_device *xe_device_get(int fd)
 {
 	struct xe_device *xe_dev, *prev;
+	int max_gt;
 
 	xe_dev = find_in_cache(fd);
 	if (xe_dev)
@@ -268,21 +370,40 @@ struct xe_device *xe_device_get(int fd)
 
 	xe_dev->fd = fd;
 	xe_dev->config = xe_query_config_new(fd);
+	xe_dev->hwconfig = xe_query_hwconfig_new(fd, &xe_dev->hwconfig_size);
 	xe_dev->va_bits = xe_dev->config->info[DRM_XE_QUERY_CONFIG_VA_BITS];
 	xe_dev->dev_id = xe_dev->config->info[DRM_XE_QUERY_CONFIG_REV_AND_DEVICE_ID] & 0xffff;
 	xe_dev->gt_list = xe_query_gt_list_new(fd);
+
+	/* GT IDs may be non-consecutive; keep a mask of valid IDs */
+	for (int gt = 0; gt < xe_dev->gt_list->num_gt; gt++)
+		xe_dev->gt_mask |= (1ull << xe_dev->gt_list->gt_list[gt].gt_id);
+
+	/* Tile IDs may be non-consecutive; keep a mask of valid IDs */
+	for (int gt = 0; gt < xe_dev->gt_list->num_gt; gt++)
+		xe_dev->tile_mask |= (1ull << xe_dev->gt_list->gt_list[gt].tile_id);
+
 	xe_dev->memory_regions = __memory_regions(xe_dev->gt_list);
 	xe_dev->engines = xe_query_engines(fd);
 	xe_dev->mem_regions = xe_query_mem_regions_new(fd);
+	xe_dev->eu_stall = xe_query_eu_stall_new(fd);
 	xe_dev->oa_units = xe_query_oa_units_new(fd);
-	xe_dev->vram_size = calloc(xe_dev->gt_list->num_gt, sizeof(*xe_dev->vram_size));
-	xe_dev->visible_vram_size = calloc(xe_dev->gt_list->num_gt, sizeof(*xe_dev->visible_vram_size));
-	for (int gt = 0; gt < xe_dev->gt_list->num_gt; gt++) {
-		xe_dev->vram_size[gt] = gt_vram_size(xe_dev->mem_regions,
-						     xe_dev->gt_list, gt);
-		xe_dev->visible_vram_size[gt] =
-			gt_visible_vram_size(xe_dev->mem_regions,
-					     xe_dev->gt_list, gt);
+
+	/*
+	 * vram_size[] and visible_vram_size[] are indexed by uapi ID; ensure
+	 * the allocation is large enough to hold the highest GT ID
+	 */
+	max_gt = igt_fls(xe_dev->gt_mask) - 1;
+	xe_dev->vram_size = calloc(max_gt + 1, sizeof(*xe_dev->vram_size));
+	xe_dev->visible_vram_size = calloc(max_gt + 1, sizeof(*xe_dev->visible_vram_size));
+
+	for (int idx = 0; idx < xe_dev->gt_list->num_gt; idx++) {
+		struct drm_xe_gt *gt = &xe_dev->gt_list->gt_list[idx];
+
+		xe_dev->vram_size[gt->gt_id] =
+			gt_vram_size(xe_dev->mem_regions, gt);
+		xe_dev->visible_vram_size[gt->gt_id] =
+			gt_visible_vram_size(xe_dev->mem_regions, gt);
 	}
 	xe_dev->default_alignment = __mem_default_alignment(xe_dev->mem_regions);
 	xe_dev->has_vram = __mem_has_vram(xe_dev->mem_regions);
@@ -380,6 +501,20 @@ _TYPE _NAME(int fd)			\
  */
 xe_dev_FN(xe_number_gt, gt_list->num_gt, unsigned int);
 
+/**
+ * xe_max_gt:
+ * @fd: xe device fd
+ *
+ * Return maximum GT ID in xe device's GT list.
+ */
+unsigned int xe_dev_max_gt(int fd)
+{
+	struct xe_device *xe_dev = find_in_cache(fd);
+
+	igt_assert(xe_dev);
+	return igt_fls(xe_dev->gt_mask) - 1;
+}
+
 /**
  * all_memory_regions:
  * @fd: xe device fd
@@ -401,6 +536,32 @@ uint64_t system_memory(int fd)
 	return regions & 0x1;
 }
 
+/*
+ * Given a uapi GT ID, lookup the corresponding drm_xe_gt structure in the
+ * GT list.
+ */
+const struct drm_xe_gt *drm_xe_get_gt(struct xe_device *xe_dev, int gt_id)
+{
+	for (int i = 0; i < xe_dev->gt_list->num_gt; i++)
+		if (xe_dev->gt_list->gt_list[i].gt_id == gt_id)
+			return &xe_dev->gt_list->gt_list[i];
+
+	return NULL;
+}
+
+/*
+ * Given a uapi GT ID, lookup the corresponding drm_xe_gt structure in the
+ * GT list and return valid tile_id otherwise invalid.
+ */
+int xe_get_tile(struct xe_device *xe_dev, int gt_id)
+{
+	for (int i = 0; i < xe_dev->gt_list->num_gt; i++)
+		if (xe_dev->gt_list->gt_list[i].gt_id == gt_id)
+			return xe_dev->gt_list->gt_list[i].tile_id;
+
+	return -ENOENT;
+}
+
 /**
  * vram_memory:
  * @fd: xe device fd
@@ -414,9 +575,9 @@ uint64_t vram_memory(int fd, int gt)
 
 	xe_dev = find_in_cache(fd);
 	igt_assert(xe_dev);
-	igt_assert(gt >= 0 && gt < xe_dev->gt_list->num_gt);
+	igt_assert(xe_dev->gt_mask & BIT(gt));
 
-	return xe_has_vram(fd) ? native_region_for_gt(xe_dev->gt_list, gt) : 0;
+	return xe_has_vram(fd) ? native_region_for_gt(drm_xe_get_gt(xe_dev, gt)) : 0;
 }
 
 static uint64_t __xe_visible_vram_size(int fd, int gt)
@@ -636,7 +797,7 @@ static void __available_vram_size_snapshot(int fd, int gt, struct __available_vr
 	xe_dev = find_in_cache(fd);
 	igt_assert(xe_dev);
 
-	region_idx = ffs(native_region_for_gt(xe_dev->gt_list, gt)) - 1;
+	region_idx = ffsll(native_region_for_gt(drm_xe_get_gt(xe_dev, gt))) - 1;
 	mem_region = &xe_dev->mem_regions->mem_regions[region_idx];
 
 	if (XE_IS_CLASS_VRAM(mem_region)) {
@@ -731,6 +892,26 @@ bool xe_has_engine_class(int fd, uint16_t engine_class)
 	return false;
 }
 
+/**
+ * xe_find_engine_by_class
+ * @fd: xe device fd
+ * @engine_class: engine class
+ *
+ * Returns engine info of xe device @fd and @engine_class otherwise NULL.
+ */
+struct drm_xe_engine *xe_find_engine_by_class(int fd, uint16_t engine_class)
+{
+	struct xe_device *xe_dev;
+
+	xe_dev = find_in_cache(fd);
+	igt_assert(xe_dev);
+
+	for (int i = 0; i < xe_dev->engines->num_engines; i++)
+		if (xe_dev->engines->engines[i].instance.engine_class == engine_class)
+			return &xe_dev->engines->engines[i];
+
+	return NULL;
+}
 /**
  * xe_has_media_gt:
  * @fd: xe device fd
@@ -751,25 +932,48 @@ bool xe_has_media_gt(int fd)
 	return false;
 }
 
+/**
+ * xe_gt_type:
+ * @fd: xe device fd
+ * @gt: gt id
+ *
+ * Returns the type of @gt for device @fd (e.g.,
+ * DRM_XE_QUERY_GT_TYPE_MAIN, DRM_XE_QUERY_GT_TYPE_MEDIA).
+ */
+uint16_t xe_gt_type(int fd, int gt)
+{
+	struct xe_device *xe_dev = find_in_cache(fd);
+	const struct drm_xe_gt *xe_gt;
+
+	igt_assert(xe_dev);
+	xe_gt = drm_xe_get_gt(xe_dev, gt);
+	igt_assert(xe_gt);
+
+	return xe_gt->type;
+}
+
 /**
  * xe_is_media_gt:
  * @fd: xe device fd
  * @gt: gt id
  *
- * Returns true if @gt for device @fd is media GT, otherwise false.
+ * Returns true if @gt for device @fd is MEDIA GT, otherwise false.
  */
 bool xe_is_media_gt(int fd, int gt)
 {
-	struct xe_device *xe_dev;
+	return xe_gt_type(fd, gt) == DRM_XE_QUERY_GT_TYPE_MEDIA;
+}
 
-	xe_dev = find_in_cache(fd);
-	igt_assert(xe_dev);
-	igt_assert(gt < xe_number_gt(fd));
-
-	if (xe_dev->gt_list->gt_list[gt].type == DRM_XE_QUERY_GT_TYPE_MEDIA)
-		return true;
-
-	return false;
+/**
+ * xe_is_main_gt:
+ * @fd: xe device fd
+ * @gt: gt id
+ *
+ * Returns true if @gt for device @fd is MAIN GT, otherwise false.
+ */
+bool xe_is_main_gt(int fd, int gt)
+{
+	return xe_gt_type(fd, gt) == DRM_XE_QUERY_GT_TYPE_MAIN;
 }
 
 /**
@@ -791,6 +995,46 @@ uint16_t xe_gt_get_tile_id(int fd, int gt)
 	return xe_dev->gt_list->gt_list[gt].tile_id;
 }
 
+/**
+ * xe_hwconfig_lookup_value:
+ * @fd: xe device fd
+ * @attribute: hwconfig attribute id
+ * @len: pointer to store length of the value (in uint32_t sized elements)
+ *
+ * Returns a pointer to the value of the hwconfig attribute @attribute and
+ * writes the number of uint32_t elements indicating the length of the value to @len.
+ */
+uint32_t *xe_hwconfig_lookup_value(int fd, enum intel_hwconfig attribute, uint32_t *len)
+{
+	struct xe_device *xe_dev;
+	uint32_t *hwconfig;
+	uint32_t pos, hwconfig_len;
+
+	xe_dev = find_in_cache(fd);
+	igt_assert(xe_dev);
+
+	hwconfig = xe_dev->hwconfig;
+	if (!hwconfig)
+		return NULL;
+
+	/* Extract the value from the hwconfig */
+	pos = 0;
+	hwconfig_len = xe_dev->hwconfig_size / sizeof(uint32_t);
+	while (pos + 2 < hwconfig_len) {
+		uint32_t attribute_id = hwconfig[pos];
+		uint32_t attribute_len = hwconfig[pos + 1];
+		uint32_t *attribute_data = &hwconfig[pos + 2];
+
+		if (attribute_id == attribute) {
+			*len = attribute_len;
+			return attribute_data;
+		}
+		pos += 2 + attribute_len;
+	}
+
+	return NULL;
+}
+
 igt_constructor
 {
 	xe_device_cache_init();
