@@ -30,7 +30,6 @@
  * Description: Testing EDID with a Chamelium board
  * Driver requirement: i915, xe
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include <fcntl.h>
@@ -38,6 +37,7 @@
 #include <string.h>
 #include <stdatomic.h>
 #include <xf86drmMode.h>
+#include "drmtest.h"
 
 #include "config.h"
 #include "igt.h"
@@ -54,33 +54,27 @@
 /**
  * SUBTEST: dp-edid-read
  * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Functionality: chamelium, dp, edid
  *
  * SUBTEST: hdmi-edid-read
  * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Functionality: chamelium, hdmi, edid
  *
  * SUBTEST: vga-edid-read
  * Description: Make sure the EDID exposed by KMS is the same as the screen's
- * Functionality: chamelium, vga, edid
  *
  * SUBTEST: dp-edid-resolution-list
  * Description: Get an EDID with many modes of different configurations, set
  *              them on the screen and check the screen resolution matches the
  *              mode resolution.
- * Functionality: chamelium, dp, edid
  *
  * SUBTEST: dp-mode-timings
  * Description: For each mode of the IGT base EDID, perform a modeset and check
  *              the mode detected by the Chamelium receiver matches the mode we
  *              set
- * Functionality: chamelium, dp, edid
  *
  * SUBTEST: hdmi-mode-timings
  * Description: For each mode of the IGT base EDID, perform a modeset and check
  *              the mode detected by the Chamelium receiver matches the mode we
  *              set
- * Functionality: chamelium, dp, edid
  */
 
 /**
@@ -88,13 +82,11 @@
  * Description: Stress test the DUT by testing multiple EDIDs, one right after
  *              the other, and ensure their validity by check the real screen
  *              resolution vs the advertised mode (%arg[1]) resolution.
- * Functionality: chamelium, dp, edid
  *
  * SUBTEST: hdmi-edid-stress-resolution-%s
  * Description: Stress test the DUT by testing multiple EDIDs, one right after
  *              the other, and ensure their validity by check the real screen
  *              resolution vs the advertised mode (%arg[1]) resolution.
- * Functionality: chamelium, hdmi, edid
  *
  * arg[1]:
  *
@@ -107,13 +99,11 @@
  * Description: Simulate a screen being unplugged and another screen being
  *              plugged during suspend, check that a uevent is sent and
  *              connector status is updated during %arg[1]
- * Functionality: chamelium, dp, edid
  *
  * SUBTEST: hdmi-edid-change-during-%s
  * Description: Simulate a screen being unplugged and another screen being
  *              plugged during suspend, check that a uevent is sent and
  *              connector status is updated during %arg[1]
- * Functionality: chamelium, dp, edid
  *
  * arg[1]:
  *
@@ -121,7 +111,9 @@
  * @suspend:      suspend
  */
 
+#define CI_LIMIT 25
 #define MODE_CLOCK_ACCURACY 0.05 /* 5% */
+static bool extended;
 
 static void get_connectors_link_status_failed(chamelium_data_t *data,
 					      bool *link_status_failed)
@@ -264,14 +256,22 @@ static const char igt_edid_stress_resolution_desc[] =
 static void edid_stress_resolution(chamelium_data_t *data,
 				   struct chamelium_port *port,
 				   monitor_edid edids_list[],
-				   size_t edids_list_len)
+				   size_t edids_list_len,
+				   bool limit)
 {
 	int i;
 	struct chamelium *chamelium = data->chamelium;
 	struct udev_monitor *mon = igt_watch_uevents();
+
 	chamelium_reset_state(&data->display, data->chamelium, port,
 			      data->ports, data->port_count);
 
+	/* TODO: Make changes to check if we can get rid of extended flag and explore
+	 * if there is environment variable in CI when executing with igt_runner
+	 */
+
+	if (!extended && is_intel_device(data->drm_fd) && limit)
+		edids_list_len = CI_LIMIT;
 
 	for (i = 0; i < edids_list_len; ++i) {
 		struct chamelium_edid *chamelium_edid;
@@ -408,12 +408,12 @@ static void test_suspend_resume_edid_change(chamelium_data_t *data,
 	chamelium_reset_state(&data->display, data->chamelium, port,
 			      data->ports, data->port_count);
 
-	/* Catch the event and flush all remaining ones. */
-	igt_assert(igt_hotplug_detected(mon, CHAMELIUM_HOTPLUG_TIMEOUT));
 	igt_flush_uevents(mon);
 
 	/* First plug in the port */
 	chamelium_set_edid(data, port, edid);
+	igt_assert(chamelium_reprobe_connector(&data->display, data->chamelium,
+		   port) == DRM_MODE_DISCONNECTED);
 	chamelium_plug(data->chamelium, port);
 	igt_assert(igt_hotplug_detected(mon, CHAMELIUM_HOTPLUG_TIMEOUT));
 
@@ -504,8 +504,22 @@ static void test_mode_timings(chamelium_data_t *data,
 	} while (++i < count_modes);
 }
 
+static int opt_handler(int opt, int opt_index, void *_data)
+{
+	switch (opt) {
+	case 'e':
+		extended = true;
+		break;
+	}
+
+	return IGT_OPT_HANDLER_SUCCESS;
+}
+
+const char *help_str =
+	"  -e \tExtended tests.\n";
+
 IGT_TEST_DESCRIPTION("Testing EDID with a Chamelium board");
-igt_main
+igt_main_args("e", NULL, help_str, opt_handler, NULL)
 {
 	chamelium_data_t data;
 	struct chamelium_port *port;
@@ -535,13 +549,13 @@ igt_main
 		igt_describe(igt_edid_stress_resolution_desc);
 		connector_subtest("dp-edid-stress-resolution-4k", DisplayPort)
 			edid_stress_resolution(&data, port, DP_EDIDS_4K,
-					       ARRAY_SIZE(DP_EDIDS_4K));
+					       ARRAY_SIZE(DP_EDIDS_4K), false);
 
 		igt_describe(igt_edid_stress_resolution_desc);
 		connector_subtest("dp-edid-stress-resolution-non-4k",
 				  DisplayPort)
 			edid_stress_resolution(&data, port, DP_EDIDS_NON_4K,
-					       ARRAY_SIZE(DP_EDIDS_NON_4K));
+					       ARRAY_SIZE(DP_EDIDS_NON_4K), false);
 
 		igt_describe(igt_edid_resolution_list_desc);
 		connector_subtest("dp-edid-resolution-list", DisplayPort)
@@ -589,12 +603,12 @@ igt_main
 		igt_describe(igt_edid_stress_resolution_desc);
 		connector_subtest("hdmi-edid-stress-resolution-4k", HDMIA)
 			edid_stress_resolution(&data, port, HDMI_EDIDS_4K,
-					       ARRAY_SIZE(HDMI_EDIDS_4K));
+					       ARRAY_SIZE(HDMI_EDIDS_4K), false);
 
 		igt_describe(igt_edid_stress_resolution_desc);
 		connector_subtest("hdmi-edid-stress-resolution-non-4k", HDMIA)
 			edid_stress_resolution(&data, port, HDMI_EDIDS_NON_4K,
-					       ARRAY_SIZE(HDMI_EDIDS_NON_4K));
+					       ARRAY_SIZE(HDMI_EDIDS_NON_4K), true);
 
 		igt_describe(test_suspend_resume_edid_change_desc);
 		connector_subtest("hdmi-edid-change-during-suspend", HDMIA)
