@@ -15,6 +15,7 @@
 #include <string.h>
 
 #include "igt.h"
+#include "linux_scaffold.h"
 #include "xe_drm.h"
 #include "xe/xe_ioctl.h"
 #include "xe/xe_query.h"
@@ -50,7 +51,7 @@ void dump_hex_debug(void *buffer, int len)
 /* Please reflect intel_hwconfig_types.h changes below
  * static_asserti_value + get_hwconfig_name
  *   Thanks :-) */
-static_assert(INTEL_HWCONFIG_MAX_GSC + 1 == __INTEL_HWCONFIG_KEY_LIMIT, "");
+static_assert(INTEL_HWCONFIG_NUM_XECU + 1 == __INTEL_HWCONFIG_KEY_LIMIT, "");
 
 #define CASE_STRINGIFY(A) case INTEL_HWCONFIG_##A: return #A;
 const char* get_hwconfig_name(int param)
@@ -137,6 +138,8 @@ const char* get_hwconfig_name(int param)
 	CASE_STRINGIFY(MIN_MESH_URB_ENTRIES);
 	CASE_STRINGIFY(MAX_MESH_URB_ENTRIES);
 	CASE_STRINGIFY(MAX_GSC);
+	CASE_STRINGIFY(SYNC_NUM_RT_STACKS_PER_DSS);
+	CASE_STRINGIFY(NUM_XECU);
 	}
 	igt_assert_lt(param, __INTEL_HWCONFIG_KEY_LIMIT);
 	igt_assert(!"Missing config table enum");
@@ -181,10 +184,8 @@ const char *get_topo_name(int value)
  * Test category: functionality test
  *
  * SUBTEST: multigpu-query-engines
- * Description: Display engine classes available for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Display engine classes available for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -195,9 +196,9 @@ test_query_engines(int fd)
 
 	xe_for_each_engine(fd, hwe) {
 		igt_assert(hwe);
-		igt_info("engine %d: %s, engine instance: %d, tile: TILE-%d\n", i++,
-			 xe_engine_class_string(hwe->engine_class), hwe->engine_instance,
-								    hwe->gt_id);
+		igt_info("engine %d: %s, engine instance: %d, gt: GT-%d, tile: TILE-%d\n",
+			i++, xe_engine_class_string(hwe->engine_class),
+			hwe->engine_instance, hwe->gt_id, xe_gt_get_tile_id(fd, hwe->gt_id));
 	}
 
 	igt_assert_lt(0, i);
@@ -205,15 +206,13 @@ test_query_engines(int fd)
 
 /**
  * SUBTEST: query-mem-usage
- * Test category: functionality test
  * Description: Display memory information like memory class, size
  *	and alignment.
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-mem-usage
- * Description: Display memory information for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Display memory information for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -272,14 +271,12 @@ test_query_mem_regions(int fd)
 
 /**
  * SUBTEST: query-gt-list
- * Test category: functionality test
  * Description: Display information about available GT components for xe device.
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-gt-list
- * Description: Display information about GT components for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Display information about GT components for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -349,14 +346,12 @@ test_query_gt_list(int fd)
 
 /**
  * SUBTEST: query-topology
- * Test category: functionality test
  * Description: Display topology information of GT.
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-topology
- * Description: Display topology information of GT for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Display topology information of GT for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -402,7 +397,7 @@ test_query_gt_topology(int fd)
 	}
 
 	/* sanity check EU type */
-	if (IS_PONTEVECCHIO(dev_id) || AT_LEAST_GEN(dev_id, 20)) {
+	if (IS_PONTEVECCHIO(dev_id) || intel_gen(dev_id) >= 20) {
 		igt_assert(topo_types & (1 << DRM_XE_TOPO_SIMD16_EU_PER_DSS));
 		igt_assert_eq(topo_types & (1 << DRM_XE_TOPO_EU_PER_DSS), 0);
 	} else {
@@ -415,13 +410,13 @@ test_query_gt_topology(int fd)
 
 /**
  * SUBTEST: query-topology-l3-bank-mask
- * Test category: functionality test
  * Description: Check the value of the l3 bank mask
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-topology-l3-bank-mask
- * Test category: functionality test
- * Description: Check the value of the l3 bank mask for all Xe devices.
  * Sub-category: MultiGPU
+ * Description: Check the value of the l3 bank mask for all Xe devices.
+ * Test category: functionality test
  */
 static void
 test_query_gt_topology_l3_bank_mask(int fd)
@@ -487,14 +482,12 @@ test_query_gt_topology_l3_bank_mask(int fd)
 
 /**
  * SUBTEST: query-config
- * Test category: functionality test
  * Description: Display xe device id, revision and configuration.
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-config
- * Description: Display config information for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Display config information for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -530,6 +523,9 @@ test_query_config(int fd)
 	igt_info("  DRM_XE_QUERY_CONFIG_FLAG_HAS_VRAM\t%s\n",
 		config->info[DRM_XE_QUERY_CONFIG_FLAGS] &
 		DRM_XE_QUERY_CONFIG_FLAG_HAS_VRAM ? "ON":"OFF");
+	igt_info("  DRM_XE_QUERY_CONFIG_FLAG_HAS_LOW_LATENCY\t%s\n",
+		 config->info[DRM_XE_QUERY_CONFIG_FLAGS] &
+		 DRM_XE_QUERY_CONFIG_FLAG_HAS_LOW_LATENCY ? "ON":"OFF");
 	igt_info("DRM_XE_QUERY_CONFIG_MIN_ALIGNMENT\t\t%#llx\n",
 		config->info[DRM_XE_QUERY_CONFIG_MIN_ALIGNMENT]);
 	igt_info("DRM_XE_QUERY_CONFIG_VA_BITS\t\t\t%llu\n",
@@ -543,14 +539,12 @@ test_query_config(int fd)
 
 /**
  * SUBTEST: query-hwconfig
- * Test category: functionality test
  * Description: Display hardware configuration of xe device.
+ * Test category: functionality test
  *
  * SUBTEST: multigpu-query-hwconfig
- * Description: Display hardware configuration for all Xe devices.
- * Category: Core
  * Sub-category: MultiGPU
- * Mega feature: General Core features
+ * Description: Display hardware configuration for all Xe devices.
  * Test category: functionality test
  */
 static void
@@ -584,14 +578,12 @@ test_query_hwconfig(int fd)
 
 /**
  * SUBTEST: query-invalid-query
- * Test category: negative test
  * Description: Check query with invalid arguments returns expected error code.
+ * Test category: negative test
  *
  * SUBTEST: multigpu-query-invalid-query
- * Description: Check query with invalid arguments for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Check query with invalid arguments for all Xe devices.
  * Test category: negative test
  */
 static void
@@ -609,14 +601,12 @@ test_query_invalid_query(int fd)
 
 /**
  * SUBTEST: query-invalid-size
- * Test category: negative test
  * Description: Check query with invalid size returns expected error code.
+ * Test category: negative test
  *
  * SUBTEST: multigpu-query-invalid-size
- * Description: Check query with invalid size for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Check query with invalid size for all Xe devices.
  * Test category: negative test
  */
 static void
@@ -634,14 +624,12 @@ test_query_invalid_size(int fd)
 
 /**
  * SUBTEST: query-invalid-extension
- * Test category: negative test
  * Description: Check query with invalid extension returns expected error code.
+ * Test category: negative test
  *
  * SUBTEST: multigpu-query-invalid-extension
- * Description: Check query with invalid extension for all Xe devices.
- * Category: Core
- * Mega feature: General Core features
  * Sub-category: MultiGPU
+ * Description: Check query with invalid extension for all Xe devices.
  * Test category: negative test
  */
 static void
@@ -687,18 +675,13 @@ query_engine_cycles(int fd, struct drm_xe_query_engine_cycles *resp)
 static uint32_t
 __engine_reference_clock(int fd, int gt_id)
 {
-	uint32_t reference_clock = 0;
 	struct xe_device *xe_dev = xe_device_get(fd);
+	const struct drm_xe_gt *gt = drm_xe_get_gt(xe_dev, gt_id);
 
-	for (int gt = 0; gt < xe_dev->gt_list->num_gt; gt++) {
-		if (gt == gt_id) {
-			reference_clock = xe_dev->gt_list->gt_list[gt].reference_clock;
-			break;
-		}
-	}
-	igt_assert(reference_clock);
+	igt_assert(gt);
+	igt_assert(gt->reference_clock);
 
-	return reference_clock;
+	return gt->reference_clock;
 }
 
 static void
@@ -706,12 +689,12 @@ __engine_cycles(int fd, struct drm_xe_engine_class_instance *hwe)
 {
 	struct drm_xe_query_engine_cycles ts1 = {};
 	struct drm_xe_query_engine_cycles ts2 = {};
-	uint64_t delta_cpu, delta_cs, delta_delta;
+	uint64_t delta_cpu, delta_cs, delta_delta, calc_freq;
 	unsigned int exec_queue;
 	int i, usable = 0;
 	igt_spin_t *spin;
 	uint64_t ahnd;
-	uint32_t vm, eng_ref_clock1, eng_ref_clock2;
+	uint32_t vm, eng_ref_clock;
 	struct {
 		int32_t id;
 		const char *name;
@@ -736,6 +719,7 @@ __engine_cycles(int fd, struct drm_xe_engine_class_instance *hwe)
 #define NUM_SNAPSHOTS 10
 	for (i = 0; i < NUM_SNAPSHOTS * ARRAY_SIZE(clock); i++) {
 		int index = i / NUM_SNAPSHOTS;
+		uint64_t width_mask;
 
 		ts1.eci = *hwe;
 		ts1.clockid = clock[index].id;
@@ -744,38 +728,41 @@ __engine_cycles(int fd, struct drm_xe_engine_class_instance *hwe)
 		ts2.clockid = clock[index].id;
 
 		query_engine_cycles(fd, &ts1);
-		eng_ref_clock1 = __engine_reference_clock(fd, hwe->gt_id);
 		query_engine_cycles(fd, &ts2);
-		eng_ref_clock2 = __engine_reference_clock(fd, hwe->gt_id);
+		eng_ref_clock = __engine_reference_clock(fd, hwe->gt_id);
 
 		igt_debug("[1] cpu_ts before %llu, reg read time %llu\n",
 			  ts1.cpu_timestamp,
 			  ts1.cpu_delta);
-		igt_debug("[1] engine_ts %llu, freq %u Hz, width %u\n",
-			  ts1.engine_cycles, eng_ref_clock1, ts1.width);
+		igt_debug("[1] engine_ts %llu, width %u\n",
+			  ts1.engine_cycles, ts1.width);
 
 		igt_debug("[2] cpu_ts before %llu, reg read time %llu\n",
 			  ts2.cpu_timestamp,
 			  ts2.cpu_delta);
-		igt_debug("[2] engine_ts %llu, freq %u Hz, width %u\n",
-			  ts2.engine_cycles, eng_ref_clock2, ts2.width);
+		igt_debug("[2] engine_ts %llu, width %u\n",
+			  ts2.engine_cycles, ts2.width);
 
 		delta_cpu = ts2.cpu_timestamp - ts1.cpu_timestamp;
 
-		if (ts2.engine_cycles >= ts1.engine_cycles)
-			delta_cs = (ts2.engine_cycles - ts1.engine_cycles) *
-				   NSEC_PER_SEC / eng_ref_clock1;
-		else
-			delta_cs = (((1 << ts2.width) - ts2.engine_cycles) + ts1.engine_cycles) *
-				   NSEC_PER_SEC / eng_ref_clock1;
+		igt_assert_eq(ts1.width, ts2.width);
+		width_mask = GENMASK_ULL(ts1.width - 1, 0);
+		ts1.engine_cycles &= width_mask;
+		ts2.engine_cycles &= width_mask;
+		delta_cs = ((ts2.engine_cycles - ts1.engine_cycles) & width_mask) *
+			NSEC_PER_SEC / eng_ref_clock;
 
-		igt_debug("delta_cpu[%lu], delta_cs[%lu]\n",
+		calc_freq = (ts2.engine_cycles - ts1.engine_cycles) * NSEC_PER_SEC / delta_cpu;
+
+		igt_debug("freq %u Hz, calc_freq %"PRIu64" Hz, err %.3f%%\n", eng_ref_clock,
+			  calc_freq, fabs((double)calc_freq - eng_ref_clock) * 100 / eng_ref_clock);
+		igt_debug("delta_cpu[%"PRIu64"], delta_cs[%"PRIu64"]\n",
 			  delta_cpu, delta_cs);
 
 		delta_delta = delta_cpu > delta_cs ?
 			       delta_cpu - delta_cs :
 			       delta_cs - delta_cpu;
-		igt_debug("delta_delta %lu\n", delta_delta);
+		igt_debug("delta_delta %"PRIu64"\n", delta_delta);
 
 		if (delta_delta < 5000)
 			usable++;
@@ -1070,6 +1057,70 @@ static void test_query_oa_units(int fd)
 	}
 }
 
+/**
+ * SUBTEST: query-pxp-status
+ * Description: Display PXP supported types and current status
+ *
+ * SUBTEST: multigpu-query-pxp-status
+ * Description: Display fields for PXP unit query for all Xe devices
+ * Sub-category: MultiGPU
+ */
+static void test_query_pxp_status(int fd)
+{
+	struct drm_xe_query_pxp_status *qpxp;
+	struct drm_xe_device_query query = {
+		.extensions = 0,
+		.query = DRM_XE_DEVICE_QUERY_PXP_STATUS,
+		.size = 0,
+		.data = 0,
+	};
+	int ret;
+
+	/*
+	 * if we run this test on an older kernel that doesn't have the PXP
+	 * query, the ioctl will return -EINVAL.
+	 */
+	errno = 0;
+	ret = igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query);
+	igt_require(errno != EINVAL);
+	igt_assert_eq(ret, 0);
+
+	/* make sure the returned size is big enough */
+	igt_assert(query.size >= sizeof(*qpxp));
+
+	qpxp = malloc(query.size);
+	igt_assert(qpxp);
+
+	memset(qpxp, 0, query.size);
+
+	query.data = to_user_pointer(qpxp);
+
+	errno = 0;
+	ret = igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query);
+	if (errno == ENODEV) {
+		igt_info("PXP not supported\n");
+		free(qpxp);
+		return;
+	}
+
+	igt_assert_eq(ret, 0);
+	igt_assert_neq(qpxp->supported_session_types, 0);
+
+	switch (qpxp->status) {
+	case 0:
+		igt_info("PXP initialization still in progress\n");
+		break;
+	case 1:
+		igt_info("PXP initialization complete\n");
+		break;
+	default:
+		igt_assert_f(0, "unexpected PXP status %u\n", qpxp->status);
+	}
+
+	igt_info("PXP supported types mask 0x%x\n", qpxp->supported_session_types);
+	free(qpxp);
+}
+
 igt_main
 {
 	const struct {
@@ -1087,6 +1138,7 @@ igt_main
 		{ "query-uc-fw-version-guc", test_query_uc_fw_version_guc },
 		{ "query-uc-fw-version-huc", test_query_uc_fw_version_huc },
 		{ "query-oa-units", test_query_oa_units },
+		{ "query-pxp-status", test_query_pxp_status },
 		{ "query-invalid-cs-cycles", test_engine_cycles_invalid },
 		{ "query-invalid-query", test_query_invalid_query },
 		{ "query-invalid-size", test_query_invalid_size },
