@@ -34,9 +34,7 @@
  *              was disabled and its drawing is directly inserted on the PF by
  *              software.
  * Driver requirement: i915, xe
- * Functionality: cursor
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include "igt.h"
@@ -49,11 +47,12 @@
 /**
  * SUBTEST: cursor-dpms
  * Description: Check random placement of a cursor with DPMS.
- * Functionality: cursor, dpms
  *
  * SUBTEST: cursor-suspend
  * Description: Check random placement of a cursor with suspend.
- * Functionality: cursor, suspend
+ *
+ * SUBTEST: cursor-size-hints
+ * Description: Check that sizes declared in SIZE_HINTS are accepted.
  *
  * SUBTEST: cursor-%s
  * Description: %arg[1]
@@ -96,6 +95,16 @@
  * @max-size:              Max supported size
  */
 
+/**
+ * SUBTEST: cursor-tearing-framebuffer-change
+ * Description: Validate cursor IOCTLs tearing via framebuffer changes.
+ */
+
+/**
+ * SUBTEST: cursor-tearing-position-change
+ * Description: Validate cursor IOCTLs tearing via cursor position change.
+ */
+
 IGT_TEST_DESCRIPTION(
    "Use the display CRC support to validate cursor plane functionality. "
    "The test will position the cursor plane either fully onscreen, "
@@ -119,6 +128,11 @@ enum cursor_buffers {
 	MAXCURSORBUFFER
 };
 
+enum cursor_change {
+	FIRSTIMAGE,
+	SECONDIMAGE
+};
+
 typedef struct {
 	int x;
 	int y;
@@ -147,6 +161,7 @@ typedef struct {
 	double alpha;
 	int vblank_wait_count; /* because of msm */
 	cursorarea oldcursorarea[MAXCURSORBUFFER];
+	struct igt_fb timed_fb[2];
 } data_t;
 
 static bool extended;
@@ -475,10 +490,15 @@ static void test_crc_offscreen(data_t *data)
 static void test_crc_sliding(data_t *data)
 {
 	int i;
+	int max;
+
 	struct {
 		igt_crc_t crc[3];
 	} rounds[16] = {};
 
+	/* In simulation, we only run a few rounds to keep the test fast */
+	max = igt_run_in_simulation() ? 4 : 16;
+
 	/* Make sure cursor moves smoothly and pixel-by-pixel, and that there are
 	 * no alignment issues. Horizontal, vertical and diagonal test.
 	 */
@@ -487,7 +507,7 @@ static void test_crc_sliding(data_t *data)
 	cursor_enable(data);
 	igt_plane_set_fb(data->primary, &data->primary_fb[HWCURSORBUFFER]);
 
-	for (i = 0; i < ARRAY_SIZE(rounds); i++) {
+	for (i = 0; i < max; i++) {
 		do_single_test(data, i, 0, true, &rounds[i].crc[0]);
 		do_single_test(data, 0, i, true, &rounds[i].crc[1]);
 		do_single_test(data, i, i, true, &rounds[i].crc[2]);
@@ -495,7 +515,7 @@ static void test_crc_sliding(data_t *data)
 
 	/* SW test */
 	cursor_disable(data);
-	for (i = 0; i < ARRAY_SIZE(rounds); i++) {
+	for (i = 0; i < max; i++) {
 		do_single_test(data, i, 0, false, &rounds[i].crc[0]);
 		do_single_test(data, 0, i, false, &rounds[i].crc[1]);
 		do_single_test(data, i, i, false, &rounds[i].crc[2]);
@@ -511,6 +531,9 @@ static void test_crc_random(data_t *data)
 
 	/* Random cursor placement */
 
+	if (igt_run_in_simulation() && max > 10)
+		max = 10;
+
 	/* HW test */
 	cursor_enable(data);
 	igt_plane_set_fb(data->primary, &data->primary_fb[HWCURSORBUFFER]);
@@ -670,13 +693,85 @@ static void test_cursor_transparent(data_t *data)
 	data->alpha = 1.0;
 }
 
+static void do_timed_cursor_fb_change(data_t *data, enum cursor_change change)
+{
+	if (change == FIRSTIMAGE) {
+		igt_plane_set_fb(data->cursor, &data->timed_fb[0]);
+		igt_plane_set_position(data->cursor,
+				       data->left + data->cursor_max_w - 10,
+				       data->bottom - data->cursor_max_h - 10);
+	} else {
+		igt_plane_set_fb(data->cursor, &data->timed_fb[1]);
+	}
+}
+
+static void do_timed_cursor_fb_pos_change(data_t *data, enum cursor_change change)
+{
+	if (change == FIRSTIMAGE) {
+		igt_plane_set_fb(data->cursor, &data->timed_fb[0]);
+		igt_plane_set_position(data->cursor,
+				       data->left + data->cursor_max_w - 10,
+				       data->bottom - data->cursor_max_h - 10);
+	} else {
+		igt_plane_set_position(data->cursor,
+				       data->left + data->cursor_max_w + 20,
+				       data->bottom - data->cursor_max_h + 20);
+	}
+}
+
+static void timed_cursor_changes(data_t *data, void (changefunc)(data_t *, enum cursor_change))
+{
+	igt_crc_t crc1, crc2;
+
+	/* Legacy cursor API does not guarantee that the cursor update happens at vBlank.
+	 * So, not assuming that this happens across all platforms, the test requires
+	 * Intel plaforms.
+	 */
+	igt_require_intel(data->drm_fd);
+
+	data->cursor = igt_output_get_plane_type(data->output, DRM_PLANE_TYPE_CURSOR);
+	changefunc(data, FIRSTIMAGE);
+
+	igt_display_commit(&data->display);
+
+	/* Extra vblank wait is because nonblocking cursor ioctl */
+	igt_wait_for_vblank_count(data->drm_fd,
+				  data->display.pipes[data->pipe].crtc_offset,
+				  data->vblank_wait_count);
+
+	igt_pipe_crc_get_current(data->drm_fd, data->pipe_crc, &crc1);
+
+	/* get the screen refresh rate, then wait for vblank, and
+	 * wait for 1/5 of time of screen refresh and change image.
+	 * change it mid screen to validate that the change happens
+	 * at the end of the current frame.
+	 */
+	usleep(1.0f / data->refresh / 5.0f * 1e6);
+
+	changefunc(data, SECONDIMAGE);
+	igt_display_commit(&data->display);
+	igt_pipe_crc_get_current(data->drm_fd, data->pipe_crc, &crc2);
+
+	igt_assert_crc_equal(&crc1, &crc2);
+}
+
+static void test_crc_cursors(data_t *data)
+{
+	timed_cursor_changes(data, do_timed_cursor_fb_change);
+}
+
+static void test_crc_pos_cursors(data_t *data)
+{
+	timed_cursor_changes(data, do_timed_cursor_fb_pos_change);
+}
+
 static void test_cursor_opaque(data_t *data)
 {
 	data->alpha = 1.0;
 	test_cursor_alpha(data);
 }
 
-static bool require_cursor_size(data_t *data, int w, int h)
+static bool cursor_size_supported(data_t *data, int w, int h)
 {
 	igt_fb_t primary_fb;
 	drmModeModeInfo *mode;
@@ -718,7 +813,7 @@ static bool require_cursor_size(data_t *data, int w, int h)
 	igt_remove_fb(data->drm_fd, &primary_fb);
 	igt_output_set_pipe(output, PIPE_NONE);
 
-	return !!ret;
+	return ret == 0;
 }
 
 static void run_test(data_t *data, void (*testfunc)(data_t *), int cursor_w, int cursor_h)
@@ -817,6 +912,52 @@ static bool execution_constraint(enum pipe pipe)
 	return false;
 }
 
+static void test_size_hints(data_t *data)
+{
+	const struct drm_plane_size_hint *hints;
+	drmModePropertyBlobPtr blob;
+	uint64_t blob_id;
+	int count;
+
+	igt_require(igt_plane_has_prop(data->cursor, IGT_PLANE_SIZE_HINTS));
+
+	blob_id = igt_plane_get_prop(data->cursor, IGT_PLANE_SIZE_HINTS);
+	/*
+	 * blob_id==0 is reserved for potential future use, but the
+	 * meaning has not yet been defined so fail outright if we see it.
+	 */
+	igt_assert(blob_id);
+
+	blob = drmModeGetPropertyBlob(data->drm_fd, blob_id);
+	igt_assert(blob);
+
+	hints = blob->data;
+	count = blob->length / sizeof(hints[0]);
+	igt_assert_lt(0, count);
+
+	for (int i = 0; i < count; i++) {
+		int w = hints[i].width;
+		int h = hints[i].height;
+
+		igt_create_fb(data->drm_fd, w, h,
+			      DRM_FORMAT_ARGB8888,
+			      DRM_FORMAT_MOD_LINEAR,
+			      &data->fb);
+
+		/*
+		 * TODO
+		 * This only confirms the kernel accepts a cursor
+		 * of this size. Verify that the cusrsor also works
+		 * correctly, if not already covered by other subtests.
+		 */
+		igt_assert(cursor_size_supported(data, w, h));
+
+		igt_remove_fb(data->drm_fd, &data->fb);
+	}
+
+	drmModeFreePropertyBlob(blob);
+}
+
 static void run_size_tests(data_t *data, int w, int h)
 {
 	enum pipe pipe;
@@ -875,7 +1016,7 @@ static void run_size_tests(data_t *data, int w, int h)
 				if (!valid_pipe_output_combo(data))
 					continue;
 
-				if (require_cursor_size(data, w, h)) {
+				if (!cursor_size_supported(data, w, h)) {
 					igt_info("Cursor size %dx%d not supported by driver\n", w, h);
 					continue;
 				}
@@ -963,6 +1104,57 @@ static void run_tests_on_pipe(data_t *data)
 	}
 
 	igt_fixture {
+		igt_create_color_fb(data->drm_fd, data->cursor_max_w, data->cursor_max_h,
+				DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR,
+				1.f, 1.f, 1.f, &data->timed_fb[0]);
+
+		igt_create_color_fb(data->drm_fd, data->cursor_max_w, data->cursor_max_h,
+				DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR,
+				1.f, 0.f, 0.f, &data->timed_fb[1]);
+	}
+
+	igt_describe("Validate cursor updates don't cause tearing with framebuffer changes");
+	igt_subtest_with_dynamic("cursor-tearing-framebuffer-change") {
+		for_each_pipe_with_single_output(&data->display, pipe, data->output) {
+			if (execution_constraint(pipe))
+				continue;
+
+			data->pipe = pipe;
+
+			if (!valid_pipe_output_combo(data))
+				continue;
+
+			igt_dynamic_f("pipe-%s-%s",
+					  kmstest_pipe_name(pipe),
+					  data->output->name)
+				run_test(data, test_crc_cursors,
+					  data->cursor_max_w, data->cursor_max_h);
+		}
+	}
+
+	igt_describe("Validate cursor updates don't cause tearing with position changes");
+	igt_subtest_with_dynamic("cursor-tearing-position-change") {
+		for_each_pipe_with_single_output(&data->display, pipe, data->output) {
+			if (execution_constraint(pipe))
+				continue;
+
+			data->pipe = pipe;
+
+			if (!valid_pipe_output_combo(data))
+				continue;
+
+			igt_dynamic_f("pipe-%s-%s",
+					  kmstest_pipe_name(pipe),
+					  data->output->name)
+				run_test(data, test_crc_pos_cursors,
+					  data->cursor_max_w, data->cursor_max_h);
+		}
+	}
+
+	igt_fixture {
+		igt_remove_fb(data->drm_fd, &data->timed_fb[0]);
+		igt_remove_fb(data->drm_fd, &data->timed_fb[1]);
+
 		create_cursor_fb(data, data->cursor_max_w, data->cursor_max_h);
 	}
 
@@ -1011,6 +1203,25 @@ static void run_tests_on_pipe(data_t *data)
 	igt_fixture
 		igt_remove_fb(data->drm_fd, &data->fb);
 
+	igt_describe("Check that sizes declared in SIZE_HINTS are accepted.");
+	igt_subtest_with_dynamic("cursor-size-hints") {
+		for_each_pipe_with_single_output(&data->display, pipe, data->output) {
+			if (execution_constraint(pipe))
+				continue;
+
+			data->pipe = pipe;
+
+			if (!valid_pipe_output_combo(data))
+				continue;
+
+			igt_dynamic_f("pipe-%s-%s",
+				      kmstest_pipe_name(pipe),
+				      data->output->name)
+				run_test(data, test_size_hints,
+					 data->cursor_max_w, data->cursor_max_h);
+		}
+	}
+
 	for (cursor_size = 32; cursor_size <= 512; cursor_size *= 2) {
 		int w = cursor_size;
 		int h = cursor_size;
